import { useReducer } from 'react';
function createReducer() {
    return function (state, action) {
        switch (action.type) {
            case 'enqueue': {
                state.queue.push(...action.values);
                break;
            }
            case 'dequeue': {
                state.queue.shift();
                break;
            }
            case 'clear': {
                return { queue: [] };
            }
            default:
                break;
        }
        return {
            queue: state.queue,
        };
    };
}
export function useQueue(arr = []) {
    const [state, dispatch] = useReducer(createReducer(), {
        queue: arr,
    });
    return {
        enqueue: (...values) => {
            dispatch({ type: 'enqueue', values });
            return values;
        },
        dequeue: () => {
            const item = state.queue[0] || null;
            dispatch({ type: 'dequeue' });
            return item;
        },
        clear: () => dispatch({ type: 'clear' }),
        forEach: (callback) => state.queue.forEach(callback),
        map: (callback) => state.queue.map(callback),
        includes: (value) => state.queue.includes(value),
        peek: () => state.queue[0] || null,
        entries: () => state.queue.entries(),
        values: () => state.queue.values(),
        keys: () => state.queue.keys(),
        size: state.queue.length,
    };
}
//# sourceMappingURL=useQueue.js.map